import { IDictionary, IEventParam, Loader, Metadata, Singleton } from '@zeedhi/core';
import {
	IButtonEvent,
	ITreeClickEvent,
	ITreeNodeModel,
	Modal,
	ModalService,
	Tree,
} from '@zeedhi/common';
import { PmLibDialogService, PmLibFormSave, PmLibHttp, PmLibAlertService } from '@zeedhi/pm-lib';
import { saveAs } from 'file-saver';
import Messages from '@/services/Messages';
import { NodeType } from '@/services/AclLabels/NodeType';
import { NodeTypeService } from '@/services/AclLabels/NodeTypeService';
import Store from '@/store';
import { AclLabelsEditLabelModalController } from '@/controllers/AclLabels/AclLabelsEditLabelModalController';
import { AclLabelsService } from '@/services/AclLabels/AclLabelsService';
import { ZdText } from '@zeedhi/vuetify';

@Singleton
export class AclLabelsController {
	/**
	 * Properties aclLabelsTree
	 */
	get originalNodes() {
		return Store.getters['AclLabels/getOriginalNodes'];
	}

	set originalNodes(value: any) {
		Store.commit('AclLabels/setOriginalNodes', value);
	}

	public hasNoSelectedNodes: boolean = true;

	public hasMultipleSelectedNodes: boolean = false;

	public saveLog: boolean = true;

	get hasChanges(): boolean {
		return Store.getters['AclLabels/getHasChanges'];
	}

	set hasChanges(value: boolean) {
		Store.commit('AclLabels/setHasChanges', value);
	}

	get version(): number {
		return Store.getters['AclLabels/getVersion'];
	}

	set version(value: number) {
		Store.commit('AclLabels/setVersion', value);
	}

	get aclLabelsTreeComponent(): Tree {
		return Store.getters['AclLabels/getAclLabelsTreeComponent'];
	}

	get nodeTypeValue(): string {
		return Store.getters['AclLabels/getNodeTypeValue'];
	}

	set nodeTypeValue(value: string) {
		Store.commit('AclLabels/setNodeTypeValue', value);
	}

	get addLabelModal(): Modal {
		return Store.getters['AclLabels/getAddLabelModal'];
	}

	get editLabelModal(): Modal {
		return Store.getters['AclLabels/getEditLabelModal'];
	}

	get importMenuModal(): Modal {
		return Store.getters['AclLabels/getImportMenuModal'];
	}

	get duplicateLabelsModal(): Modal {
		return Store.getters['AclLabels/getDuplicateLabelsModal'];
	}

	set currentNodeEdit(node: any) {
		Store.commit('AclLabels/setCurrentNodeEdit', node);
	}

	public getNodeTitle(node: ITreeNodeModel<any>) {
		return node.title;
	}

	public getNodeVersion(node: ITreeNodeModel<any>) {
		if (node.data.version) {
			return `v${node.data.version}`;
		}
		return 'Novo';
	}

	public getNodeVersionClass(node: ITreeNodeModel<any>) {
		return node.data.version ? '' : 'button-font-color-orange-new-label';
	}

	public getNodeVersionIsVisible(node: ITreeNodeModel<any>) {
		return !!node.data.version;
	}

	public getNodeLabel(node: ITreeNodeModel<any>) {
		return node.data.label;
	}

	public getVisibleButtonCopySlot(node: ITreeNodeModel<any>): boolean {
		return !!node.data.label;
	}

	private async getAclLabels() {
		const projectId = Store.getters['AclLabels/getCurrentProject'].PROJECT_ID;
		await Store.dispatch('AclLabels/loadNodes', projectId);
	}

	public getNodeTypeLabel(node: ITreeNodeModel<any>) {
		return NodeTypeService.getNodeTypeLabel(node.data.type);
	}

	public getNodeTypeColor(node: ITreeNodeModel<any>) {
		return NodeTypeService.getNodeTypeColor(node.data.type);
	}

	/**
	 * Events methods aclLabelsTree
	 */

	public async onCreatedAclLabelsTree() {
		await Store.dispatch('AclLabels/loadAddLabelModal');
		await Store.dispatch('AclLabels/loadEditLabelModal');
		this.openFileModal = await ModalService.createFromJson(this.openFileModalPath, true);
		await Store.dispatch('AclLabels/loadImportMenuModal');
		await Store.dispatch('AclLabels/loadDuplicateLabelsModal');
	}

	public onBeforeDestroyAclLabelsTree() {
		this.addLabelModal?.destroy();
		this.editLabelModal?.destroy();
		this.openFileModal?.destroy();
		this.importMenuModal?.destroy();
		this.duplicateLabelsModal?.destroy();
	}

	public onNodeSelectAclLabelsTree({ component }: IEventParam<Tree>) {
		this.hasNoSelectedNodes = component.selectedNodes.length === 0 || this.hasNodeMenuType(component.selectedNodes);
		this.hasMultipleSelectedNodes = this.hasMultipleSelected(component.selectedNodes);
	}

	private hasMultipleSelected(selectedNodes: any) {
		if (selectedNodes.length > 1) {
			return true;
		}

		if (selectedNodes.length === 1) {
			const node = selectedNodes[0];
			return this.nodeIsRuleType(node);
		}

		return false;
	}

	private nodeIsRuleType(node: ITreeNodeModel<any>) {
		return node.data?.type === 'RULE';
	}

	private hasNodeMenuType(nodes: Array<ITreeNodeModel<any>>): boolean {
		return nodes.some((node) => node.data.type === NodeType.menu);
	}

	public async onNodeDblClickAclLabelsTree({ node }: ITreeClickEvent) {
		await this.setNodeTypeValueByUpdate(node);
	}

	private async setNodeTypeValueByUpdate(node: ITreeNodeModel<any>) {
		if (node.data.type === NodeType.grouped) {
			this.nodeTypeValue = NodeType.grouped;
			await this.loadEditLabelModal(node);
		} else if (node.data.type === NodeType.rule) {
			this.nodeTypeValue = NodeType.rule;
			await this.loadEditLabelModal(node);
		}
	}

	private async loadEditLabelModal(node: any) {
		this.currentNodeEdit = node;
		await this.editLabelModal?.show();
		const editForm = Metadata.getInstance<PmLibFormSave>('editLabelModalForm');
		const aclLabelsEditLabelModalController: AclLabelsEditLabelModalController = Loader.getInstance('AclLabelsEditLabelModalController');
		aclLabelsEditLabelModalController.editNode = node;
		aclLabelsEditLabelModalController.editNodeBackup = node;
		editForm.value.name = node.title;
		editForm.value.label = node.data?.label;
	}

	/**
	 * Actions methods aclLabelsTree
	 */

	public async clickButtonCopySlot({ component }: IButtonEvent) {
		try {
			const labelComponentName = component.parent!.children[0].name;
			const zdTextLabel = Metadata.getInstance<ZdText>(labelComponentName);
			const text = typeof zdTextLabel.text === 'string' ? zdTextLabel.text : zdTextLabel.text[0];
			await navigator.clipboard.writeText(text);
		} catch (error) {
			PmLibDialogService.showError(error);
		}
	}

	public clickAclLabelsTreeButtonAdd() {
		const nodes = this.aclLabelsTreeComponent.selectedNodes;
		if (nodes.length && nodes.length === 1) {
			const node = nodes[0];
			if (node.data?.type === NodeType.grouped || node.data?.type === NodeType.menu) {
				this.nodeTypeValue = NodeType.rule;
				this.addLabelModal?.show();
			}
		} else if (!nodes.length) {
			this.nodeTypeValue = NodeType.grouped;
			this.addLabelModal?.show();
		}
	}

	public async clickAclLabelsTreeButtonDelete() {
		const response = await PmLibDialogService.confirmDialog('Deseja excluir o(s) Label(s) selecionado(s)');
		if (response) {
			this.aclLabelsTreeComponent.removeSelected();
			this.hasMultipleSelectedNodes = false;
			await this.clickAclLabelsTreeButtonSave();
		}
	}

	public async clickAclLabelsTreeButtonReload() {
		await this.getAclLabels();
		this.hasChanges = false;
		this.aclLabelsTreeComponent.clearSelection();
		this.hasNoSelectedNodes = true;
		this.hasMultipleSelectedNodes = false;
	}

	public clickAclLabelsTreeButtonExpand() {
		this.setIsExpandedAllNodes(this.aclLabelsTreeComponent.nodes, true);
	}

	public clickAclLabelsTreeButtonCollapse() {
		this.setIsExpandedAllNodes(this.aclLabelsTreeComponent.nodes, false);
	}

	private setIsExpandedAllNodes(nodes: Array<ITreeNodeModel<any>>, value: boolean) {
		nodes.forEach((node) => {
			node.isExpanded = value;
			if (node.children?.length) {
				this.setIsExpandedAllNodes(node.children, value);
			}
		});
	}

	public clickAclLabelsTreeButtonCancel() {
		if (this.originalNodes) {
			this.aclLabelsTreeComponent.nodes = [...this.originalNodes];
		} else {
			this.aclLabelsTreeComponent.nodes = [];
		}
		this.hasChanges = false;
		this.aclLabelsTreeComponent.clearSelection();
	}

	public async clickAclLabelsTreeButtonSave() {
		await this.saveNodes();
	}

	private async saveNodes(saveLog: boolean = true) {
		const value = this.aclLabelsTreeComponent.nodes;
		this.hasChanges = false;
		this.originalNodes = [...value];
		this.aclLabelsTreeComponent.clearSelection();
		const labelsFileString = JSON.stringify(this.originalNodes);
		const projectId = Store.getters['AclLabels/getCurrentProject'].PROJECT_ID;
		const data: any = { labelsFile: labelsFileString, projectId, saveLog };
		await PmLibHttp.$post('/projects/aclLabels', data);
		PmLibAlertService.showSuccess(Messages.SUCCESS_SAVE_DEAFULT);
	}

	public clickAclLabelsTreeButtonOpenFile() {
		this.openFileModal?.show();
	}

	public async clickAclLabelsTreeButtonDownload() {
		let nameArchive;
		const { nodes } = this.aclLabelsTreeComponent;

		const majorVersion = AclLabelsService.getMajorVersion(nodes);
		this.setNodesVersion(nodes, majorVersion);
		this.setLabelParent(nodes);

		try {
			const openFileForm = Metadata.getInstance<PmLibFormSave>('openFileModalForm');
			nameArchive = openFileForm.value.fileJson[0].name;
		} catch (error) {
			nameArchive = 'aclLabels.json';
		}
		const blob = new Blob([JSON.stringify(nodes)], { type: 'text/plain;charset=utf-8' });
		saveAs(blob, nameArchive);
		this.hasChanges = false;
		this.aclLabelsTreeComponent.nodes = nodes;
		await this.saveNodes(this.saveLog);
		this.saveLog = true;
		await this.getAclLabels();
	}

	private setNodesVersion(nodes: Array<ITreeNodeModel<any>>, majorVersion: number) {
		this.saveLog = false;
		nodes.forEach((node) => {
			node.data.checked = false;
			node.data.new = false;
			if (!node.data.version) {
				this.saveLog = true;
				node.data.version = majorVersion + 1;
			}

			if (node.children?.length) {
				this.setNodesVersion(node.children, majorVersion);
			}
		});
	}

	private setLabelParent(nodes: Array<ITreeNodeModel<any>>) {
		nodes.forEach((node) => {
			if (node.children?.length) {
				const parent = node.data.type === NodeType.grouped ? node.title : node.data.label;
				this.setLabelParentChildren(node.children, parent);
			}
		});
	}

	private setLabelParentChildren(nodesChildren: Array<ITreeNodeModel<any>>, labelParent: string) {
		nodesChildren.forEach((node) => {
			node.data.parent = labelParent;
			if (node.children?.length) {
				const parent = node.data.type === NodeType.grouped ? node.title : node.data.label;
				this.setLabelParentChildren(node.children, parent);
			}
		});
	}

	public async clickAclLabelsTreeButtonImportMenu() {
		this.aclLabelsTreeComponent.clearSelection();
		this.importMenuModal?.show();
	}

	/**
	 * Open File Modal Controller
	 */

	public openFileModal!: Modal;

	private openFileModalPath = 'aclLabels/components/detail/tabs/labels/modais/openFile';

	private errorMsgFileOpen = 'Permitidos apenas arquivos .json válidos';

	public closeOpenFileModalForm() {
		this.openFileModal?.hide();
	}

	public changeInputFileJson() {
		try {
			const openFileForm = Metadata.getInstance<PmLibFormSave>('openFileModalForm');
			const readFile = openFileForm.value.fileJson;
			const reader = new FileReader();

			reader.onload = (evt: any) => {
				try {
					const nodesString = evt.target.result.replaceAll('"isSelected":true', '"isSelected":false');
					this.aclLabelsTreeComponent.nodes = JSON.parse(nodesString);
					this.version = this.getLastVersion();
					this.hasNoSelectedNodes = true;
				} catch (e) {
					this.hasChanges = false;
					PmLibDialogService.showError(this.errorMsgFileOpen);
				}
			};

			if (openFileForm.validate()) {
				reader.readAsText(readFile);
				this.openFileModal?.hide();
				this.hasChanges = true;
			}
		} catch (error) {
			this.hasChanges = false;
			PmLibDialogService.showError(this.errorMsgFileOpen);
		}
	}

	/**
	 * Aux
	 */

	private getLastVersion() {
		let version = 0;
		this.aclLabelsTreeComponent.nodes.forEach((node) => {
			if (node.data && node.data.version > version) {
				// eslint-disable-next-line prefer-destructuring
				version = node.data.version;
			}
			version = this.getLastChildrenVersion(node, version);
		});

		return version;
	}

	private getLastChildrenVersion(node: ITreeNodeModel<IDictionary<any>>, version: number) {
		let childrenVersion = version;
		if (node.children) {
			node.children.forEach((child) => {
				if (child.data && child.data.version > childrenVersion) {
					childrenVersion = child.data.version;
				}

				childrenVersion = this.getLastChildrenVersion(child, childrenVersion);
			});
		}
		return childrenVersion;
	}
}
